/* rt.h */
/* definitions for rt programs excluding model parameters */
/* 93/08/24 merged from independent files */


/* ========================= constant values ============================= */
#define LINE   0        /* here is molecular envelope */
#define CONT   1        /* here is continuum region */

#define LVG    0        /* calculation with LVG approximation */
#define NLVG   1        /* calculation without LVG approximation */

#define LTE    0        /* calculation with LTE assumptions */
#define NLTE   1        /* calculation with non-LTE assumptions */

#define TA     0        /* output data in Ta (K) */
#define JY     1        /* output data in Jy */

/* ======================= variable type definition ====================== */
typedef struct AXIS_STRUCT {
   int n;            /* number of elements */
   int rpix;         /* x = rval + (n - rpix) * delt */
   double rval;
   double delt;
} AXIS;

/* ------------------------------ in trans.c ----------------------------- */
typedef struct LOS_STRUCT {
   int n;         /* number of elements */
   int *lc;       /* status at the position (0: molecule, 1: continuum) */
   double *x;     /* x position */
   double *v;     /* velocity at x */
   double *turb;  /* turblance velocity at x */
   double *dvdr;  /* dv / dr */
   double *ngu;   /* n / G of higher state */
   double *ngl;   /* n / G of lower state */
} LOS;

/* allocate LOS type variable with n elements */
#define InitLOS(los, nn) { \
   los.n    = (nn); \
   los.lc   = (int *)calloc(los.n, sizeof(int)); \
   los.x    = (double *)calloc(los.n, sizeof(double)); \
   los.v    = (double *)calloc(los.n, sizeof(double)); \
   los.turb = (double *)calloc(los.n, sizeof(double)); \
   los.dvdr = (double *)calloc(los.n, sizeof(double)); \
   los.ngu  = (double *)calloc(los.n, sizeof(double)); \
   los.ngl  = (double *)calloc(los.n, sizeof(double)); \
} /* InitLOS() */

/* ============================= in basic.c ============================== */
extern double Doppler();
extern double F2V();
extern double BB();
extern double ItoTb();
extern double ItoJy();

/* ============================= in rate.c =============================== */
extern double RateEq();

/* ============================= in trans.c ============================== */
extern void Trans();

/* ===========================in convolv.c =============================== */
extern void MakeBeam();
extern void MakeBeam2D();
extern void MakeBeamOffset();
extern void ImConv();
extern void SpConv();
extern void SpConv2D();
extern void FqBinning();

/* =========================== in output.c =============================== */
void WriteOneSp();
void WriteMap();

/* ======================================================================= */
/* end of rt.h */
