/* funcs.c  */
/*    general useful functions */
/*    programmed by I.Yamamura */
/*    combined 92/01/24 */
/* 93/01/09 some functions are declined as void function. */
/* 94/04/16 spline routines are custermized to be faster */
/*          SplinePreProcess() will not be called automatically */
/*          from make_sp_table() */
/* 94/04/21 strupper(), strlower() inplimented */

#include <stdio.h>
#include <math.h>
#include <malloc.h> /* 92/08/12 */
#include <ctype.h>  /* 94/04/21 */

#include "funcs.h"

/* -------------------------------------------------------------- */
/*                   Memory allocation routine                    */
/* -------------------------------------------------------------- */
double **d_alloc(m, n)
/* allocate 2-dimensional array, double x[m][n] */
   int m, n;
{
   double **x;
   int i;

   x = (double **)calloc((unsigned)m, (unsigned)sizeof(double *));
   for (i = 0; i < m; i++)
      x[i] = (double *)calloc((unsigned)n, (unsigned)sizeof(double));

   return x;
} /* d_alloc */

void d_free(x, m)
/* free 2-dimensional array, double x[m][] */
   double **x;
   int m;
{
   int i;

   for (i = 0; i < m; i++)
      free((char *)x[i]);
   free((char *)x);
} /* d_free */

/* ====== 3D array ====== */
double ***t_alloc(l, m, n)
/* allocate 3-dimensional array, double x[m][n] */
   int l, m, n;
{
   double ***x;
   int i, j;

   x = (double ***)calloc((unsigned)l, (unsigned)sizeof(double **));
   for (i = 0; i < l; i++) {
      x[i] = (double **)calloc((unsigned)m, (unsigned)sizeof(double *));
         for (j = 0; j < m; j++)
            x[i][j] = (double *)calloc((unsigned)n, (unsigned)sizeof(double));
   }
   return x;
} /* t_alloc */

void t_free(x, l, m)
/* free 3-dimensional array, double x[l][m][] */
   double ***x;
   int l, m;
{
   int i, j;

   for (i = 0; i < l; i++) {
      for (j = 0; j < m; j++)
         free((char *)x[i][j]);
      free((char *)x[i]);
   }
   free((char *)x);

} /* t_free */

/* ===== Spline interpolation routine ===== */
/* ----- Pre process for the making spline table ----- */
void SplinePreProcess(n, x, y)
   int n;
   double *x, *y;
{
   double tx, ty;   /* working area */
   int i, j;

/* If the data seems to be in decreasing order. Reverse it. */
   if (x[0] > x[n-1]) {
      for (i = 0; i < n/2; i++) {
         tx = x[i]; x[i] = x[n-1-i]; x[n-1-i] = tx;
         ty = y[i]; y[i] = y[n-1-i]; y[n-1-i] = ty;
      }
   }

/* Sort the data in increasing order. As the data seems to be almost */
/* in right order, we use insertion sort algorithm. */
   for (i = 1; i < n; i++) {
      tx = x[i];  ty = y[i];
      for (j = i - 1; j >= 0 && x[j] > tx; j--) {
         x[j + 1] = x[j];  y[j + 1] = y[j];
      }
      x[j + 1] = tx; y[j + 1] = ty;
   }
} /* SplinePreProcess() */

/* ===== First of all, make table for spline() ===== */

void make_sp_table(n, x, y, z)
   int n;
   double *x, *y, *z;
{
   int i;
   double t;
   double *h, *d;

   h = (double *)calloc((unsigned)n, (unsigned)sizeof(double));
   d = (double *)calloc((unsigned)n, (unsigned)sizeof(double));

   z[0] = z[n-1] = 0;
   for (i = 0; i < n - 1; i++) {
      h[i] = x[i+1] - x[i];
      d[i+1] = (y[i+1] - y[i]) / h[i];
   }
   z[1] = d[2] - d[1] - h[0] * z[0];
   d[1] = 2 * (x[2] - x[0]);
   for (i = 1; i < n - 2; i++) {
      t = h[i] / d[i];
      z[i+1] = d[i+2] - d[i+1] - z[i] * t;
      d[i+1] = 2 * (x[i+2] - x[i]) - h[i] * t;
   }
   z[n-2] -= h[n-2] * z[n-1];
   for (i = n - 2; i > 0; i--)
      z[i] = (z[i] - h[i] * z[i+1]) / d[i];

   free(h); free(d);
} /* maketable () */

double spline(t, n, x, y, z)
   int n;                  /* number of data */
   double t, *x, *y, *z;   /* t: input x, x,y: data, z: work area */
{
   int i, j, k;
   static int ib = -1;
   static double e, f, g, h;
   double d, r;

/* this algorithm may be faster than below when t come in order. */
/*
   i = 0;
   j = n - 1;
   if (ib > 0 && t < x[ib])    j = ib;     else i = ib + 1;
   if (ib > 0 && t > x[ib+1])  i = ib + 1; else j = ib + 1;
   while (i < j) {
      k = (i + j) / 2;
      if (x[k] < t) i = k + 1; else j = k;
   }
   if (i > 0) i--;

   d = t - x[i];
   if (i == ib) {
      r =  ((f * d / h + z[i] * 3.0) * d + (g / h - e * h)) * d + y[i];
   } else {
      ib = i;
      h = x[i+1] - x[i];
      g = y[i+1] - y[i];
      f = z[i+1] - z[i];
      e = z[i] * 2.0 + z[i+1];
      r =  ((f * d / h + z[i] * 3.0) * d + (g / h - e * h)) * d + y[i];
   }

   return r;
/**/

/* here is the original formulae. they may be faster than above ones */
/* when t comes romdomly */

   i = 0;
   j = n - 1;
   while (i < j) {
      k = (i + j) / 2;
      if (x[k] < t) i = k + 1; else j = k;
   }
   if (i > 0) i--;
   h = x[i+1] - x[i];
   d = t - x[i];
   r =  (((z[i+1] - z[i]) * d / h + z[i] * 3.0) * d + ((y[i+1] - y[i]) / h
        - (z[i] * 2.0 + z[i+1]) * h)) * d + y[i];

   return r;
/**/
} /* spline() */

/* ===== 2-Dimensional Spline interpolation routine ===== */
/* The coordinate (x, y) must be on the grid. */
/* 93/02/01 */
void make_sp_table2D(m, n, x, y, z, w)
   int m, n;      /* dimension of x and y */
   double *x;     /* x[m]: position of x */ 
   double *y;     /* y[n]: position of y */ 
   double **z;    /* z[m][n]: value at (x, y) */
   double **w;    /* working area w[m][n] */
{
   int i, j;

   for (i = 0; i < m; i++)
      make_sp_table(n, y, z[i], w[i]);

} /* maketable2D () */

double spline2D(tx, ty, m, n, x, y, z, w, w2, w3)
   double tx, ty; /* position to be interpolated */
   int m, n;      /* dimension of x and y */
   double *x;     /* x[m]: position of x */
   double *y;     /* y[n]: position of y */
   double **z;    /* z[m][n]: value at (x, y) */
   double **w;    /* working area w[m][n] */
   double *w2;    /* working area w2[m] */
   double *w3;    /* working area w3[m] */
{
   int i, j;
   double r;            /* returned value */

   for (i = 0; i < m; i++)
      w2[i] = spline(ty, n, y, z[i], w[i]);

   make_sp_table(m, x, w2, w3);

/*   for (i = 0; i < m; i++)
         printf("%d %d %e %e %e %e\n", i, m, tx, x[i], w2[i], w3[i]);
/**/
   r = spline(tx, m, x, w2, w3);

   return r;
} /* spline2D() */

/* ===== 3-Dimensional Spline interpolation routine ===== */
/* The coordinate (x, y, z) must be on the grid. */
/* 93/02/04 */
void make_sp_table3D(l, m, n, x, y, z, v, w)             
   int l, m, n;   /* dimension of x and y */             
   double *x;     /* x[l]: position of x */              
   double *y;     /* y[m]: position of y */              
   double *z;     /* z[n]: position of y */              
   double ***v;   /* v[l][m][n]: value at (x, y, z) */   
   double ***w;   /* working area w[l][m][n] */          
{
   int i, j;

   for (i = 0; i < l; i++)
      for (j = 0; j < m; j++)
         make_sp_table(n, z, v[i][j], w[i][j]);

} /* maketable3D () */

double spline3D(tx, ty, tz, l, m, n, x, y, z, v, w, w2, w3, w4, w5)
   double tx, ty, tz; /* position to be interpolated */
   int l, m, n;   /* dimension of x and y */
   double *x;     /* x[l]: position of x */ 
   double *y;     /* y[m]: position of y */ 
   double *z;     /* z[n]: position of y */ 
   double ***v;   /* v[l][m][n]: value at (x, y, z) */
   double ***w;   /* working area w[l][m][n] */
   double *w2;    /* working area w[m] */
   double *w3;    /* working area w[m] */
   double *w4;    /* working area w[l] */
   double *w5;    /* working area w[l] */
{
   int i, j, k;
   double r;            /* returned value */

   for (i = 0; i < l; i++) {
      for (j = 0; j < m; j++) {
         w2[j] = spline(tz, n, z, v[i][j], w[i][j]);
      }
      make_sp_table(m, y, w2, w3);
      w4[i] = spline(ty, m, y, w2, w3);
   }
   make_sp_table(l, x, w4, w5);
   r = spline(tx, l, x, w4, w5);

   return r;
} /* spline3D() */

double LinerInterpolate(x, n, xx, yy)
/* interploate by linear line. 93/02/23 */
   double x;
   int n;      /* size of xx and yy, n < 0 means the inverse order of xx */
   double *xx, *yy;
{
   static int FIRST = 0;
   int i;
   double r;

   if (FIRST == 0) {
      SplinePreProcess(n, xx, yy);
      FIRST = 1;
   }

   for (i = 0; (i < n) && (x > xx[i]); i++);
   if (i == 0) {
      r = exp(-(xx[0] - x)/(xx[1]-xx[0])) * yy[0];
   } else if (i == n) {
      r = exp(-(x - xx[n-1])/(xx[n-1]-xx[n-2])) * yy[n-1];
   } else {
      r = (yy[i-1] * (xx[i] - x) + yy[i] * (x - xx[i-1])) / (xx[i] - xx[i-1]);
   }

   return r;
} /* LinerInterpolate() */

/* ===== convert string to double ===== */
double atod(s)
/* convert string to double. */
   char *s;
{
   int i;

   for (i = 0; s[i] != '\0'; i++)
      if ((s[i] == 'D') || (s[i] == 'd')) s[i] = 'E';
   return atof(s);
} /* atod() */

/* ===== reast square methord ===== */
void rsm(n, x, y, m, a)
/* fit data to y = a[0] + a[1]*x + a[2]*x^2 + ... + a[n-1]*x^n-1 */
   int n;         /* number of data */
   double *x, *y; /* data to be fit */
   int m;         /* dimension */
   double *a;     /* m-th array of result */
{
   double **s, *t, *p, sum, tum;
   int im , jm, km, in;
   
   m = m + 1;
   s = d_alloc(m, m);
   t = (double *)calloc((unsigned)m, (unsigned)sizeof(double));
   p = (double *)calloc((unsigned)(2*n), (unsigned)sizeof(double));

   for (in = 0, sum = 0.0; in < n; in++)
      p[in] = 1.0;
   for (im = 0; im < 2*m; im++) {
      for (in = 0, sum = tum = 0.0; in < n; in++) {
         p[in] *= x[in];
         sum += p[in];
         tum += p[in] * y[in];
      }
      if (im < m) t[im] = tum;
      for (jm = 0; jm < m; jm++)
         for (km = 0; km < m; km++)
            if (jm+km == im) s[jm][km] = sum;
   }

   yLins(m, s, t, a);

   d_free(s, m);
   free((char *)t);
   free((char *)p);
} /* rsm() */

/* ===== Solve arbitrary function by Newton-Raphson methord. ===== */
int Newton(Func, x, lim, imax)
/* based on Algorithm Dictionary by I.Okumura with some modifications. */
/* bug fix 94/02/23 */
/* some improvement in conversion judgement 94/02/26 */
   double (*Func)();    /* function to be solved */ 
   double *x;           /* initial value */
   double lim;          /* convergence limit */
   int imax;            /* maximum limit of iteration */
{
   int i;
   double h, fx, df, x_prev, fx_prev;

   fx = Func(*x);
   while (fabs(fx) > EPSILON) {
      df = Func(*x + fx) - fx;
      if (fabs(df) < EPSILON) break;
      h = fx * fx / df;
      x_prev = *x; fx_prev = fx; i = 0;
      do {
         *x = x_prev - h;
         fx = Func(*x);
         h /= 2.0;
         i++;
      } while (fabs(fx) > fabs(fx_prev));
      if (--imax == 0) break;
      if (i == 1 && fabs((*x - x_prev)/x_prev) < lim) break;
   }

   return imax;
} /* Newton() */

int LogNewton(Func, x, lim, imax)
   double (*Func)();    /* function to be solved */ 
   double *x;           /* initial value */
   double lim;          /* convergence limit */
   int imax;            /* maximum limit of iteration */
{
   int n = 1;
   double h, fx, df, x_prev, fx_prev;
   double hh;

   fx = Func(*x);
   while (fx != 0.0) {
      df = (Func(*x + fx) - fx);
      x_prev = *x; fx_prev = fx;
      hh = 1.0 / (M_E * n++);
      if (df < 0.0) hh = -hh;
      do {
         *x = x_prev * (1.0 - hh);
         fx = Func(*x);
         hh /= 2.0;
      } while (fabs(fx) > fabs(fx_prev));
      if (--imax == 0) break;
      if (fabs((*x - x_prev)/x_prev) < lim) break;
   }

   return imax;
} /* LogNewton() */

#define samesign(x, y) (((x) > 0) == ((y) > 0))
int flspos(Func, x, a, b, tolerance, imax)
/* find root of a function Func() by fallse position methord */
/* if the root cannot be found until imax-th repeat, solve it by */
/* bisection methord. 93/02/26. based on Algorithm Dictionary */
/* by H.Okumura with some modifications */
/* bug fix for comparison with tolerance (fabs needed) 93/12/17 */
   double *x;           /* return value */
   double a, b;         /* find solution between a and b */
   double tolerance;
   double (*Func)();    /* function to be solved */
   int imax;            /* maximum repeat befor useing bisection methord.*/
{
   int i;
   double fa, fb, fx;

   if (tolerance < 0.0) tolerance = 0.0;
   if (b < a) {*x = a; a = b; b = *x; }  /* swap a and b */
   fa = Func(a); if (fa == 0.0) {*x = a; return 0;}
   fb = Func(b); if (fb == 0.0) {*x = b; return 0;}
   if (samesign(fa, fb)) return -1;
   for (i = 0; ; i++) {
      *x = (a + b) / 2.0;
      if (fabs((*x - a)/a) < tolerance || fabs((b - *x)/b) < tolerance) break;
      if (i < imax) {
         *x = (a * fb - b * fa) / (fb - fa);
         if      (*x <= a) *x = a + 0.6 * DBL_EPSILON * fabs(a);
         else if (*x >= b) *x = b - 0.6 * DBL_EPSILON * fabs(a);
      }
      fx = Func(*x); if (fx == 0.0) break;
      if (samesign(fx, fa)) { a = *x; fa = fx;}
      else                  { b = *x; fb = fx;}
   }
   return i;
} /* flspos() */

/* -------------------------------------------- */
void Rotate(x, y, X, Y, theta)
/* (X, Y) is the position on the corrdinate which is rotated by an angle */
/* theta to counter-clockwize from the coordinate which (x, y) is on */
/* 93/10/04 I.Yamamura */
   double x, y;   /* position befor rotation */
   double *X, *Y; /* position after rotation */
   double theta;  /* angle to rotate */
{
   double cs = cos(theta);
   double si = sin(theta);

   *X =  x * cs + y * si;
   *Y = -x * si + y * cs;
} /* Rotate() */

/* -------------------------------------------- */
void Car2Sph(x, y, z, r, theta, phai)
/* cartician corrdinate to spherical coordinate */
/* 93/10/04 I.Yamamura */
   double x, y, z;            /* cartician coordinate */
   double *r, *theta, *phai;  /* spherical coordinate */
{
   *r = sqrt(x * x + y * y + z * z);
   *theta = acos(z / *r);
   *phai = acos(x / (*r * sin(*theta)));
} /* Cart2Sphe() */

/* =================== string functions ========================= */
char *strupper(s)
/* translate all characters in the string upper case */
   char *s;
{
   int i;
   for (i = 0; s[i] != '\0'; i++)  s[i] = toupper(s[i]);
   return s;
} /* strupper */

char *strlower(s)
/* translate all characters in the string lower case */
   char *s;
{
   int i;
   for (i = 0; s[i] != '\0'; i++)  s[i] = tolower(s[i]);
   return s;
} /* strlower */

/* =================== convert system of angle ========================= */
void deg60(d, dd, md, sd)
/* convert degree to 60-th style */
/* 94/06/19 */
   double d;               /* degree */
   double *dd, *md, *sd;   /* returned values */
{
   *dd = (int)d;  d = (d - *dd) * 60;
   *md = (int)d;  *sd = (d - *md) * 60;
} /* deg60() */

/* end of funcs.c  */
