/* molhco+.c */
/* Initialize for HCO+ molecule */
/*    programed by I.Yamamura 92/06/17 based on molco.c */
/* Bug fix about Aij when (J-1 -> J)   92/08/22 */
/*    It's not Bug!!! recoverd   92/08/24 */
/* related to new made rtconfig.h 92/08/30 */
/*    since variables in rtvars.h are not used here, */
/*    this routine is independent to rtvars.h */
/* VJDIM and LV are defined in mol.h, so they are removed in the routines */
/*    92/09/02 */
/* 92/11/15 Totally rewrited to use struct variables */
/* 93/10/28 definition of VMAX and JMAX are changed */
/*          they represent the number of level but not the maximum level */

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <malloc.h>

#include "macro.h"
#include "const.h"
#include "funcs.h"
#include "matrix.h"

#include "mol.h"

#define i(v,j) ((v) * (JMAX) + (j))     /* array index */

#define VMAX  1               /* number of vibrational level */
#define JMAX  10              /* number of rotation level */
#define VJDIM (VMAX) * (JMAX)

#define MYU 3.3               /* [debey] dipole moment */
#define B   (44.5944e9/C)     /* [cm-1] J=1 level energy */
#define EV  3088.7            /* [cm-1] v=1 level energy */

#define CDNF 20.0        /* collision excitation rate multiple factor */
                         /* used in CalcCik(), CDNF = 1 if mol. is CO */

MOLECULE HCOp;

/* -------------------------------------------------------------- */
void HCOp_Init(HCOp)
   MOLECULE *HCOp;
{
   int v, j, vl, vu, jl, ju;

   strcpy(HCOp->name, "HCOp");

   HCOp->NV = VMAX;
   HCOp->NJ = JMAX;
   HCOp->LV = VJDIM;

/* memory allocation */
   HCOp->g =    (int *)calloc((unsigned)VJDIM, (unsigned)sizeof(int));
   HCOp->e =    (double *)calloc((unsigned)VJDIM, (unsigned)sizeof(double));
   HCOp->a =    d_alloc(VJDIM, VJDIM);
   HCOp->de =   d_alloc(VJDIM, VJDIM);
   HCOp->nyu0 = d_alloc(VJDIM, VJDIM);
   HCOp->ct =   d_alloc(VJDIM, VJDIM);

   HCOp->ng0 =  (double *)calloc((unsigned)VJDIM, (unsigned)sizeof(double));
   HCOp->ng =   (double *)calloc((unsigned)VJDIM, (unsigned)sizeof(double));

/* ----- enargy level */
   for (v = 0; v < VMAX; v++)
      for (j = 0; j < JMAX; j++)
         HCOp->e[i(v,j)] = EV * v + B * j * (j + 1);

/* ----- degeneracy */
   for (v = 0; v < VMAX; v++)
      for (j = 0; j < JMAX; j++)
         HCOp->g[i(v,j)] = 2 * j + 1;

/* ----- Av,j<->v,j-1 */
   for (vl = 0; vl < VMAX; vl++)
      for (vu = vl; vu < VMAX; vu++)
         for (jl = 0; jl < JMAX; jl++)
            for (ju = jl; ju < JMAX; ju++)

   if (vl == vu)
      if (ju == jl + 1)
         HCOp->a[i(vl,jl)][i(vu,ju)] = HCOp->a[i(vu,ju)][i(vl,jl)]
            = 3.137E-7 * ju/(double)(2*ju+1) * cub(HCOp->e[ju]-HCOp->e[jl]) * sqr(MYU);
      else
         HCOp->a[i(vl,jl)][i(vu,ju)] = HCOp->a[i(vu,ju)][i(vl,jl)] = 0.0;
   else if (vu == vl + 1)
      if (ju == jl + 1)
         HCOp->a[i(vl,jl)][i(vu,ju)] = HCOp->a[i(vu,ju)][i(vl,jl)] =
         HCOp->a[i(vu,jl)][i(vl,ju)] = HCOp->a[i(vl,ju)][i(vu,jl)]
            = 34.4 * ju / (double)(2*ju+1);
      else
         HCOp->a[i(vl,jl)][i(vu,ju)] = HCOp->a[i(vu,ju)][i(vl,jl)] =
         HCOp->a[i(vu,jl)][i(vl,ju)] = HCOp->a[i(vl,ju)][i(vu,jl)] = 0.0;
   else
      HCOp->a[i(vl,jl)][i(vu,ju)] = HCOp->a[i(vu,ju)][i(vl,jl)] =
      HCOp->a[i(vu,jl)][i(vl,ju)] = HCOp->a[i(vl,ju)][i(vu,jl)] = 0.0;

/* ----- energy difference and frequency */
   for (vl = 0; vl < VMAX; vl++)
      for (vu = 0; vu < VMAX; vu++)
         for (jl = 0; jl < JMAX; jl++)
            for (ju = 0; ju < JMAX; ju++) {
               HCOp->nyu0[i(vl,jl)][i(vu,ju)] = C*fabs(HCOp->e[i(vu,ju)]-HCOp->e[i(vl,jl)]);
               HCOp->de[i(vl,jl)][i(vu,ju)] = -(HCOp->e[i(vu,ju)]-HCOp->e[i(vl,jl)])/WK;
            }

} /* HCOp_Init() */

/* -------------------------------------------------------------- */
void HCOp_Stat(HCOp, nH2, abn, tp)
   MOLECULE *HCOp;
   double nH2, abn, tp;
{
#define QLIMIT 1.0E-15
   int v, j;
   double nsum, qj, f;
   static double eb, q, b;
   static double tsave = 0.0;

/* --- population --- */
   if (tp != tsave) {
      q = 0.0;
      j = 0;
      do {
         qj = 0.0;
         for (v = 0; v < VMAX; v++)
            qj = qj + (2*j+1) * exp(-(v*EV + B*j*(j+1))/WK/tp);
         q = q + qj;
         j++;
      } while (qj >= QLIMIT);
      tsave = tp;
   }
   HCOp->nmol = nH2 * abn;
   nsum = 0;
   for (v = 0; v < VMAX; v++) {
      for (j = 0; j < JMAX; j++) {
         HCOp->ng0[i(v,j)] = HCOp->ng[i(v,j)] =
               HCOp->nmol * exp(-HCOp->e[i(v,j)] / WK / tp) / q;
         nsum = nsum + HCOp->ng0[i(v,j)] * HCOp->g[i(v,j)];
      }
   }
/*   nmol = nsum;*/

   HCOp_Cik(HCOp, tp, nH2);

}  /* HCOp_Stat() */

/* -------------------------------------------------------------- */
void HCOp_Cik(HCOp, tp, nH2)
   MOLECULE *HCOp;
   double tp, nH2;
{
   int j, k;

   CO_Cik(HCOp, tp, nH2);
   for (k = 0; k < JMAX; k++)
      for (j = k; j < JMAX; j++) {
         HCOp->ct[j][k] *= CDNF;
         HCOp->ct[k][j] *= CDNF;
      }
} /* HCOp_Cik() */
