/* model.h */
/* header file for model.c */
/* new made (proto-type) 93/08/22 I.Yamamura */

/* ============================== parameters ============================= */
typedef struct CONFIG_STRUCT {
   char IMFLNM[256];          /* image output file name */
   char SPFLNM[256];          /* spectral output file name */

   double DISTANCE;           /* distance to the star (pc) */

   int    SNPIX;              /* number of pixel */
   int    SRPIX;              /* reference pixel no. */
   double SRVAL;              /* reference value */
   double SDELT;              /* incliment */

   int    FNPIX;              /* number of pixel */
   int    FRPIX;              /* reference pixel no. */
   double RESTFQ;             /* rest frequency (Hz) */
   double VCENT;              /* center velocity (cm -s) */
   double FWIDTH;             /* bandwidth (Hz) */

   double HPBW;               /* beamsize (arcsec) */
   double EFF;                /* main beam efficiency */
   double HPBWY;              /* beamsize (arcsec) */
   double HPBWZ;              /* beamsize (arcsec) */
   double PA;                 /* position angle (deg) */

   int    JU;                 /* upper energy level */

   int    LTE_M;              /* LTE mode */
   int    LVG_M;              /* LVG mode */

   int    NOFF;               /* number of offset */
   double OFFSET1;            /* offset of beam center (arcsec) */
   double OFFSET2;            /* offset of beam center */
   double OFFSET3;            /* offset of beam center */
   double OFFSET4;            /* offset of beam center */
} CONFIG;

typedef struct MODEL_STRUCT {
   double S_RO;               /* radius of the central star (cm) */
   double S_T;                /* Teff of the central star (K) */

   double H_RI;               /* inner radius of the ionized core (cm) */
   double H_EI;               /* elongation */
   double H_II;               /* inclination (degree) */

   double H_RO;               /* outer radius of the ionized core (cm) */
   double H_EO;               /* elongation */
   double H_IO;               /* inclination (degree) */

   double H_NE;               /* electron density (cm-3) */
   double H_T;                /* electron temperature (K) */

   double C_RI;               /* inner radius of the envelope (cm) */
   double C_EI;               /* elongation */
   double C_IC;               /* inclination (degree) */

   double C_RO;               /* outer radius of the envelope (cm) */
   double C_EO;               /* elongation */
   double C_IO;               /* inclination (degree) */

   double C_RB;               /* outer radius of bipolar region (cm) */
   double C_EB;               /* elongation */
   double C_IB;               /* inclination (degree) */

   double C_MLR;              /* mass loss rate (Mo yr-1) */
   char C_MOLNM[20];          /* name of the molecule */
   double C_ABN;              /* abundance (/H2) */
   double C_T;                /* Tk of the envelope (K) */

   double C_VSYS;             /* systemic velocity (cm s-1) */
   double C_VEXP;             /* expansion velocity (cm s-1) */
   double C_TURB;             /* tturblance velocity (cm s-1) */
} MODEL;

/* ======================================================================= */
void ReadParam();
void PrintParam();

/* ======================================================================= */
extern int Where();
extern void MolEnv();
extern void ContEnv();

/* ================== to access phyical parameters =================== */
extern double R_Star();
extern double T_Star();
extern double RO_HII();

/* model.h */
